@@ -75,8 +75,6 @@ zbyte far *pcp = NULL;
 
 static FILE *story_fp = NULL;
 
-z_header_t z_header;
-
 /*
  * Data for the undo mechanism.
  * This undo mechanism is based on the scheme used in Evin Robertson's
@@ -401,6 +399,7 @@ void init_memory(void)
 		{        TRINITY,  11, "860509" },
 		{        TRINITY,  12, "860926" },
 		{        TRINITY,  15, "870628" },
+		{     CUTTHROATS,  23, "840809" },
 		{        UNKNOWN,   0, "------" }
 	};
 	/* INDENT-ON */
