@@ -2,6 +2,19 @@
  * Various status thingies for the interpreter and interface.
  *
  */
+ 
+#ifndef COMMON_SETUP_H
+#define COMMON_SETUP_H
+
+
+typedef enum {
+	FORMAT_NORMAL,
+	FORMAT_UNKNOWN,
+	FORMAT_DISABLED,
+	FORMAT_IRC,
+	FORMAT_ANSI
+} format_t;
+
 typedef struct frotz_setup_struct {
 	int attribute_assignment;
 	int attribute_testing;
@@ -33,7 +46,8 @@ typedef struct frotz_setup_struct {
         char *zcode_path;
 	char *restricted_path;
 
-	bool restore_mode; /* for a save file passed from command line*/
+	format_t format;  /* type of format codes for dumb interface */
+	bool restore_mode; /* for a save file passed from command line */
 	bool use_blorb;
 	bool exec_in_blorb;
 } f_setup_t;
@@ -84,3 +98,5 @@ typedef struct zcode_header_struct {
 	zword x_back_colour;
 } z_header_t;
 extern z_header_t z_header;
+
+#endif
