@@ -32,6 +32,8 @@
 #include <assert.h>
 #include <unistd.h> //pread
 
+#include "ux_defines.h"
+
 #ifdef USE_NCURSES_H
 #include <ncurses.h>
 #else
@@ -43,8 +45,8 @@
 #include "ux_blorb.h"
 #include "ux_audio.h"
 
-f_setup_t f_setup;
-z_header_t z_header;
+extern f_setup_t f_setup;
+extern z_header_t z_header;
 
 #ifndef NO_SOUND
 
@@ -880,7 +882,7 @@ os_init_sound(void)
 	err = pthread_create(&unused_id, &attr, &audio_loop, NULL);
 	if (err != 0) {
 		fprintf(stderr, "Can't create audio thread :[%s]", strerror(err));
-		exit(EXIT_FAILURE);
+		os_quit(EXIT_FAILURE);
 	}
 }
 
