@@ -32,9 +32,10 @@
 
 #include <unistd.h>
 #include <ctype.h>
-#include <signal.h>
 #include <libgen.h>
 
+#include "ux_defines.h"
+
 #ifdef USE_NCURSES_H
 #include <ncurses.h>
 #else
@@ -52,8 +53,8 @@
 #include <locale.h>
 #endif
 
-f_setup_t f_setup;
-z_header_t z_header;
+extern f_setup_t f_setup;
+extern z_header_t z_header;
 
 volatile sig_atomic_t terminal_resized = 0;
 
@@ -87,8 +88,7 @@ char stripped_story_name[FILENAME_MAX+1];
 char semi_stripped_story_name[FILENAME_MAX+1];
 */
 
-f_setup_t f_setup;
-u_setup_t u_setup;
+extern u_setup_t u_setup;
 
 /* static void sigwinch_handler(int); */
 static void sigint_handler(int);
@@ -162,7 +162,7 @@ void os_fatal (const char *s, ...)
 		} else {
 			os_reset_screen();
 			ux_blorb_stop();
-			exit(EXIT_FAILURE);
+			os_quit(EXIT_FAILURE);
 		}
 	}
 
@@ -175,7 +175,7 @@ void os_fatal (const char *s, ...)
 
 	fputs ("\n\n", stderr);
 
-	exit (EXIT_FAILURE);
+	os_quit (EXIT_FAILURE);
 } /* os_fatal */
 
 /* extern char script_name[]; */
@@ -205,10 +205,10 @@ void os_process_arguments (int argc, char *argv[])
 
 	zoptarg = NULL;
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__HAIKU__)
 	if ((getuid() == 0) || (geteuid() == 0)) {
 		printf("I won't run as root!\n");
-		exit(EXIT_FAILURE);
+		os_quit(EXIT_FAILURE);
 	}
 #endif
 
@@ -220,7 +220,7 @@ void os_process_arguments (int argc, char *argv[])
 
 	if ((home = getenv(HOMEDIR)) == NULL) {
 		printf("Hard drive on fire!\n");
-		exit(EXIT_FAILURE);
+		os_quit(EXIT_FAILURE);
 	}
 
 
@@ -310,7 +310,7 @@ void os_process_arguments (int argc, char *argv[])
 		case 'S': f_setup.script_cols = atoi(zoptarg); break;
 		case 't': u_setup.tandy_bit = 1; break;
 		case 'u': f_setup.undo_slots = atoi(zoptarg); break;
-		case 'v': print_version(); exit(EXIT_SUCCESS); break;
+		case 'v': print_version(); os_quit(EXIT_SUCCESS); break;
 		case 'w': u_setup.screen_width = atoi(zoptarg); break;
 		case 'x': f_setup.expand_abbreviations = 1; break;
 		case 'Z':
@@ -334,7 +334,7 @@ void os_process_arguments (int argc, char *argv[])
 
 		puts (INFORMATION);
 		puts (INFO2);
-		exit (EXIT_SUCCESS);
+		os_quit (EXIT_SUCCESS);
 	}
 
 	/* This section is exceedingly messy and really can't be fixed
@@ -371,7 +371,7 @@ void os_process_arguments (int argc, char *argv[])
 		strncat(f_setup.save_name, EXT_SAVE, strlen(EXT_SAVE) + 1);
 	} else {  /*Set our auto load save as the name_save*/
 		f_setup.save_name = malloc((strlen(f_setup.tmp_save_name) + strlen(EXT_SAVE)) * sizeof(char) + 1);
-		memcpy(f_setup.save_name, f_setup.tmp_save_name, (strlen(f_setup.story_name) + strlen(EXT_SAVE)) * sizeof(char));
+		memcpy(f_setup.save_name, f_setup.tmp_save_name, (strlen(f_setup.tmp_save_name) + strlen(EXT_SAVE)) * sizeof(char));
 		free(f_setup.tmp_save_name);
 	}
 
@@ -452,7 +452,7 @@ void os_init_screen (void)
 
 	if (initscr() == NULL) {    /* Set up curses */
 		os_fatal("Unable to initialize curses. Maybe your $TERM setting is bad.");
-		exit(EXIT_FAILURE);
+		os_quit(EXIT_FAILURE);
 	}
 	u_setup.curses_active = 1;	/* Let os_fatal know curses is running */
 	raw();				/* Raw input mode, no line processing */
@@ -1072,13 +1072,7 @@ static void sigwinch_handler(int UNUSED(sig))
 static void sigint_handler(int UNUSED(dummy))
 {
 	signal(SIGINT, sigint_handler);
-
-	os_stop_sample(0);
-	scrollok(stdscr, TRUE);
-	scroll(stdscr);
-	refresh();
-	endwin();
-	exit(1);
+	os_quit(EXIT_FAILURE);
 } /* sigint_handler */
 
 
@@ -1138,8 +1132,7 @@ static void print_version(void)
 #else
 	printf("Audio output disabled.");
 #endif
-	printf("\nBuild date:\t%s\n", BUILD_DATE);
-	printf("Commit date:\t%s\n", GIT_DATE);
+	printf("\nCommit date:\t%s\n", GIT_DATE);
 	printf("Git commit:\t%s\n", GIT_HASH);
 	printf("  Frotz was originally written by Stefan Jokisch.\n");
 	printf("  It complies with standard 1.0 of Graham Nelson's specification.\n");
