@@ -33,6 +33,8 @@
 #include <sys/time.h>
 #include <sys/select.h>
 
+#include "ux_defines.h"
+
 #ifdef USE_NCURSES_H
 #include <ncurses.h>
 #else
@@ -47,7 +49,7 @@
 
 #ifndef NO_SOUND
 #include "ux_sema.h"
-ux_sem_t sound_done;
+extern ux_sem_t sound_done;
 #endif
 
 static int start_of_prev_word(int, const zchar*);
