@@ -39,8 +39,8 @@ zword hx_flags;
 zword hx_fore_colour;
 zword hx_back_colour;
 
-z_header_t z_header;
-bb_map_t *blorb_map;
+extern z_header_t z_header;
+extern bb_map_t *blorb_map;
 
 extern FILE *blorb_fp;
 
